'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.parseAsync = parseAsync;

var _parse = require('@mona/parse');

var _internals = require('@mona/internals');

var _combinators = require('@mona/combinators');

/**
 * Executes a parser asynchronously, returning an object that can be used to
 * manage the parser state. Unless the parser given tries to match eof(),
 * parsing will continue until the parser's done() function is called.
 *
 * @param {Function} parser - The parser to execute.
 * @param {AsyncParserCallback} callback - node-style 2-arg callback executed
 *                                         once per successful application of
 *                                         `parser`.
 * @param {Object} [opts] - Options object.
 * @param {String} [opts.fileName] - filename to use for error messages.
 * @memberof module:mona/api
 * @instance
 *
 * @example
 * var handle = parseAsync(token(), function(tok) {
 *  console.log('Got a token: ', tok)
 * })
 * handle.data('foobarbaz')
 */
function parseAsync(parser, callback) {
  var opts = arguments.length <= 2 || arguments[2] === undefined ? {} : arguments[2];

  opts = (0, _internals.copy)(opts);
  // Force the matter in case someone gets clever.
  opts.throwOnError = true;
  opts.returnState = true;
  opts.allowTrailing = true;
  var done = false;
  var buffer = '';
  function exec() {
    if (done && !buffer.length) {
      return false;
    }
    var res = undefined;
    try {
      res = (0, _parse.parse)((0, _combinators.collect)(parser, { min: 1 }), buffer, opts);
      opts.position = res.position;
      buffer = res.input.slice(res.offset);
    } catch (e) {
      if (!e.wasEof || done) {
        callback(e);
      }
      return false;
    }
    res.value.forEach(function (val) {
      return callback(null, val);
    });
    return true;
  }
  function errIfDone(cb) {
    return function () {
      if (done) {
        throw new Error('AsyncParser closed');
      } else {
        return cb.apply(undefined, arguments);
      }
    };
  }
  var handle = {
    done: errIfDone(function () {
      done = true;
      buffer = '';
      while (exec()) {}
      return handle;
    }),
    data: errIfDone(function (data) {
      buffer += data;
      while (exec()) {}
      return handle;
    }),
    error: errIfDone(function (error) {
      done = true;
      callback(error);
      return handle;
    })
  };
  return handle;
}